@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 
+__author__ = r'wsb310@gmail.com'
+
 import asyncio
 import aiomysql
 
@@ -18,10 +20,10 @@ from hagworm.extend.error import MySQLReadOnlyError, MySQLClientDestroyed
 from .base import Utils, WeakContextVar, AsyncContextManager, AsyncCirculator
 
 
-MONGO_POLL_WATER_LEVEL_WARNING_LINE = 0x08
+MONGO_POLL_WATER_LEVEL_WARNING_LINE = 0x10
 
-MYSQL_ERROR_RETRY_COUNT = 0x1f
-MYSQL_POLL_WATER_LEVEL_WARNING_LINE = 0x08
+MYSQL_ERROR_RETRY_COUNT = 0x10
+MYSQL_POLL_WATER_LEVEL_WARNING_LINE = 0x10
 
 
 class MongoPool:
@@ -30,7 +32,7 @@ class MongoPool:
 
     def __init__(
             self, host, username=None, password=None,
-            *, name=None, min_pool_size=8, max_pool_size=32, max_idle_time=3600, wait_queue_timeout=10,
+            *, name=None, auth_source=r'admin', min_pool_size=8, max_pool_size=32, max_idle_time=3600, wait_queue_timeout=10,
             compressors=r'zlib', zlib_compression_level=6,
             **settings
     ):
@@ -38,6 +40,7 @@ class MongoPool:
         self._name = name if name is not None else Utils.uuid1()[:8]
 
         settings[r'host'] = host
+        settings[r'authSource'] = auth_source
         settings[r'minPoolSize'] = min_pool_size
         settings[r'maxPoolSize'] = max_pool_size
         settings[r'maxIdleTimeMS'] = max_idle_time * 1000
@@ -257,31 +260,23 @@ class MySQLPool:
 
     def _echo_pool_info(self):
 
-        global MYSQL_POLL_WATER_LEVEL_WARNING_LINE
-
-        if (self._pool.maxsize - self._pool.size + self._pool.freesize) < MYSQL_POLL_WATER_LEVEL_WARNING_LINE:
-            Utils.log.warning(
-                f'MySQL connection pool not enough ({self._name}): '
-                f'{self._pool.freesize}({self._pool.size}/{self._pool.maxsize})'
-            )
-        else:
+        if self.health:
             Utils.log.debug(
                 f'MySQL connection pool info ({self._name}): '
                 f'{self._pool.freesize}({self._pool.size}/{self._pool.maxsize})'
             )
+        else:
+            Utils.log.warning(
+                f'MySQL connection pool not enough ({self._name}): '
+                f'{self._pool.freesize}({self._pool.size}/{self._pool.maxsize})'
+            )
 
-    async def health(self):
+    @property
+    def health(self):
 
-        result = False
+        global MYSQL_POLL_WATER_LEVEL_WARNING_LINE
 
-        async with self.get_client() as client:
-
-            proxy = await client.execute(r'select version();')
-            await proxy.close()
-
-            result = True
-
-        return result
+        return (self._pool.maxsize - self._pool.size + self._pool.freesize) > MYSQL_POLL_WATER_LEVEL_WARNING_LINE
 
     async def reset(self):
 
@@ -289,8 +284,6 @@ class MySQLPool:
 
             await self._pool.clear()
 
-            await self.health()
-
             Utils.log.info(
                 f'MySQL connection pool reset ({self._name}): {self._pool.size}/{self._pool.maxsize}'
             )
@@ -305,14 +298,20 @@ class MySQLPool:
 
     def get_client(self):
 
-        result = None
+        client = None
 
         try:
-            result = DBClient(self)
+
+            client = DBClient(self)
+
+            if not self.health:
+                client.set_auto_release(True)
+
         except Exception as err:
+
             Utils.log.exception(err)
 
-        return result
+        return client
 
     def get_transaction(self):
 
@@ -371,11 +370,24 @@ class MySQLDelegate:
 
     async def mysql_health(self):
 
-        result = await self._mysql_rw_pool.health() if self._mysql_rw_pool else True
-        result &= await self._mysql_ro_pool.health() if self._mysql_ro_pool else True
+        result = await self._check_health(self._mysql_rw_pool)
+        result &= await self._check_health(self._mysql_ro_pool)
 
         return result
 
+    async def _check_health(self, pool):
+
+        if pool is None:
+            return True
+
+        async with pool.get_client() as client:
+
+            await client.safe_execute(r'select version();')
+
+            return True
+
+        return False
+
     async def reset_mysql_pool(self):
 
         if self._mysql_rw_pool:
@@ -420,6 +432,10 @@ class MySQLDelegate:
 
                     self._mysql_ro_client_context.set(client)
 
+                else:
+
+                    Utils.log.debug(r'Share mysql ro client in context')
+
             else:
 
                 _client = self._mysql_ro_client_context.get()
@@ -430,9 +446,15 @@ class MySQLDelegate:
                 client = self._mysql_rw_client_context.get()
 
                 if client is None:
+
                     client = self._mysql_rw_pool.get_client()
+
                     self._mysql_rw_client_context.set(client)
 
+                else:
+
+                    Utils.log.debug(r'Share mysql rw client in context')
+
         return client
 
     def get_db_transaction(self):
@@ -468,9 +490,20 @@ class _ClientBase:
 
         return val
 
-    def __init__(self, readonly=False):
+    def __init__(self, pool):
 
-        self._readonly = readonly
+        self._pool = pool
+        self._conn = None
+
+        self._readonly = pool.readonly
+        self._auto_release = False
+
+        self._lock = asyncio.Lock()
+
+    def __del__(self):
+
+        if self._conn is not None:
+            Utils.log.critical(r'Mysql connection leak')
 
     @property
     def readonly(self):
@@ -480,148 +513,10 @@ class _ClientBase:
     @property
     def insert_id(self):
 
-        raise NotImplementedError()
-
-    def _get_conn(self):
-
-        raise NotImplementedError()
-
-    async def execute(self, clause):
-
-        raise NotImplementedError()
-
-    async def select(self, query, *multiparams, **params):
-
-        result = []
-
-        if not isinstance(query, Select):
-            raise TypeError(r'Not sqlalchemy.sql.selectable.Select object')
-
-        proxy = await self.execute(query, *multiparams, **params)
-
-        if proxy is not None:
-
-            records = await proxy.cursor.fetchall()
-
-            if records:
-                result.extend(records)
-
-            if not proxy.closed:
-                await proxy.close()
-
-        return result
-
-    async def find(self, query, *multiparams, **params):
-
-        result = None
-
-        if not isinstance(query, Select):
-            raise TypeError(r'Not sqlalchemy.sql.selectable.Select object')
-
-        proxy = await self.execute(query.limit(1), *multiparams, **params)
-
-        if proxy is not None:
-
-            record = await proxy.cursor.fetchone()
-
-            if record:
-                result = record
-
-            if not proxy.closed:
-                await proxy.close()
-
-        return result
-
-    async def count(self, query, *multiparams, **params):
-
-        result = await self.find(query, *multiparams, **params)
-
-        return result[r'tbl_row_count']
-
-    async def insert(self, query, *multiparams, **params):
-
-        result = 0
-
-        if self._readonly:
-            raise MySQLReadOnlyError()
-
-        if not isinstance(query, Insert):
-            raise TypeError(r'Not sqlalchemy.sql.dml.Insert object')
-
-        proxy = await self.execute(query, *multiparams, **params)
-
-        if proxy is not None:
-
-            result = self.insert_id
-
-            if not proxy.closed:
-                await proxy.close()
-
-        return result
-
-    async def update(self, query, *multiparams, **params):
-
-        result = 0
-
-        if self._readonly:
-            raise MySQLReadOnlyError()
-
-        if not isinstance(query, Update):
-            raise TypeError(r'Not sqlalchemy.sql.dml.Update object')
-
-        proxy = await self.execute(query, *multiparams, **params)
-
-        if proxy is not None:
-
-            result = proxy.rowcount
-
-            if not proxy.closed:
-                await proxy.close()
-
-        return result
-
-    async def delete(self, query, *multiparams, **params):
-
-        result = 0
-
-        if self._readonly:
-            raise MySQLReadOnlyError()
-
-        if not isinstance(query, Delete):
-            raise TypeError(r'Not sqlalchemy.sql.dml.Delete object')
-
-        proxy = await self.execute(query, *multiparams, **params)
-
-        if proxy is not None:
-
-            result = proxy.rowcount
-
-            if not proxy.closed:
-                await proxy.close()
-
-        return result
-
-
-class DBClient(_ClientBase, AsyncContextManager):
-    """MySQL客户端对象，使用with进行上下文管理
-
-    将连接委托给客户端对象管理，提高了整体连接的使用率
-
-    """
-
-    def __init__(self, pool):
-
-        super().__init__(pool.readonly)
-
-        self._lock = asyncio.Lock()
-
-        self._pool = pool
-        self._conn = None
-
-    @property
-    def insert_id(self):
-
-        return self._conn.connection.insert_id()
+        if self._conn:
+            return self._conn.connection.insert_id()
+        else:
+            return None
 
     async def _get_conn(self):
 
@@ -646,6 +541,177 @@ class DBClient(_ClientBase, AsyncContextManager):
             else:
                 await _conn.close()
 
+    async def _execute(self, clause):
+
+        raise NotImplementedError()
+
+    async def execute(self, clause):
+
+        result = None
+
+        async with self._lock:
+            result = await self._execute(clause)
+
+        return result
+
+    async def safe_execute(self, clause):
+
+        async with self._lock:
+
+            proxy = await self._execute(clause)
+            await proxy.close()
+
+            if self._auto_release:
+                await self._close_conn()
+
+    async def select(self, query, *multiparams, **params):
+
+        result = []
+
+        if not isinstance(query, Select):
+            raise TypeError(r'Not sqlalchemy.sql.selectable.Select object')
+
+        async with self._lock:
+
+            proxy = await self._execute(query, *multiparams, **params)
+
+            if proxy is not None:
+
+                records = await proxy.cursor.fetchall()
+
+                if records:
+                    result.extend(records)
+
+                if not proxy.closed:
+                    await proxy.close()
+
+            if self._auto_release:
+                await self._close_conn()
+
+        return result
+
+    async def find(self, query, *multiparams, **params):
+
+        result = None
+
+        if not isinstance(query, Select):
+            raise TypeError(r'Not sqlalchemy.sql.selectable.Select object')
+
+        async with self._lock:
+
+            proxy = await self._execute(query.limit(1), *multiparams, **params)
+
+            if proxy is not None:
+
+                record = await proxy.cursor.fetchone()
+
+                if record:
+                    result = record
+
+                if not proxy.closed:
+                    await proxy.close()
+
+            if self._auto_release:
+                await self._close_conn()
+
+        return result
+
+    async def count(self, query, *multiparams, **params):
+
+        result = await self.find(query, *multiparams, **params)
+
+        return result[r'tbl_row_count']
+
+    async def insert(self, query, *multiparams, **params):
+
+        result = 0
+
+        if self._readonly:
+            raise MySQLReadOnlyError()
+
+        if not isinstance(query, Insert):
+            raise TypeError(r'Not sqlalchemy.sql.dml.Insert object')
+
+        async with self._lock:
+
+            proxy = await self._execute(query, *multiparams, **params)
+
+            if proxy is not None:
+
+                result = self.insert_id
+
+                if not proxy.closed:
+                    await proxy.close()
+
+            if self._auto_release:
+                await self._close_conn()
+
+        return result
+
+    async def update(self, query, *multiparams, **params):
+
+        result = 0
+
+        if self._readonly:
+            raise MySQLReadOnlyError()
+
+        if not isinstance(query, Update):
+            raise TypeError(r'Not sqlalchemy.sql.dml.Update object')
+
+        async with self._lock:
+
+            proxy = await self._execute(query, *multiparams, **params)
+
+            if proxy is not None:
+
+                result = proxy.rowcount
+
+                if not proxy.closed:
+                    await proxy.close()
+
+            if self._auto_release:
+                await self._close_conn()
+
+        return result
+
+    async def delete(self, query, *multiparams, **params):
+
+        result = 0
+
+        if self._readonly:
+            raise MySQLReadOnlyError()
+
+        if not isinstance(query, Delete):
+            raise TypeError(r'Not sqlalchemy.sql.dml.Delete object')
+
+        async with self._lock:
+
+            proxy = await self._execute(query, *multiparams, **params)
+
+            if proxy is not None:
+
+                result = proxy.rowcount
+
+                if not proxy.closed:
+                    await proxy.close()
+
+            if self._auto_release:
+                await self._close_conn()
+
+        return result
+
+
+class DBClient(_ClientBase, AsyncContextManager):
+    """MySQL客户端对象，使用with进行上下文管理
+
+    将连接委托给客户端对象管理，提高了整体连接的使用率
+
+    """
+
+    def set_auto_release(self, val):
+
+        self._auto_release = val
+
     async def _context_release(self):
 
         await self._close_conn(self._lock.locked())
@@ -653,48 +719,45 @@ class DBClient(_ClientBase, AsyncContextManager):
     async def release(self):
 
         async with self._lock:
-
             await self._close_conn()
 
-    async def execute(self, query, *multiparams, **params):
+    async def _execute(self, query, *multiparams, **params):
 
         global MYSQL_ERROR_RETRY_COUNT
 
         result = None
 
-        async with self._lock:
+        async for times in AsyncCirculator(max_times=MYSQL_ERROR_RETRY_COUNT):
 
-            async for times in AsyncCirculator(max_times=MYSQL_ERROR_RETRY_COUNT):
+            try:
 
-                try:
+                conn = await self._get_conn()
 
-                    conn = await self._get_conn()
+                result = await conn.execute(query, *multiparams, **params)
 
-                    result = await conn.execute(query, *multiparams, **params)
+            except (Warning, DataError, IntegrityError, ProgrammingError) as err:
 
-                except (Warning, DataError, IntegrityError, ProgrammingError) as err:
+                await self._close_conn(True)
 
-                    await self._close_conn(True)
+                raise err
 
+            except Exception as err:
+
+                await self._close_conn(True)
+
+                if times < MYSQL_ERROR_RETRY_COUNT:
+                    Utils.log.exception(err)
+                else:
                     raise err
 
-                except Exception as err:
+            else:
 
-                    await self._close_conn(True)
-
-                    if times < MYSQL_ERROR_RETRY_COUNT:
-                        Utils.log.exception(err)
-                    else:
-                        raise err
-
-                else:
-
-                    break
+                break
 
         return result
 
 
-class DBTransaction(DBClient):
+class DBTransaction(_ClientBase, AsyncContextManager):
     """MySQL客户端事务对象，使用with进行上下文管理
 
     将连接委托给客户端对象管理，提高了整体连接的使用率
@@ -709,7 +772,7 @@ class DBTransaction(DBClient):
 
     async def _get_conn(self):
 
-        if self._conn is None:
+        if self._trx is None:
             self._conn = await super()._get_conn()
             self._trx = await self._conn.begin()
 
@@ -717,8 +780,6 @@ class DBTransaction(DBClient):
 
     async def _close_conn(self, discard=False):
 
-        await super()._close_conn(discard)
-
         if self._trx is not None:
 
             if self._trx.is_active:
@@ -726,6 +787,8 @@ class DBTransaction(DBClient):
 
             self._trx = None
 
+        await super()._close_conn(discard)
+
         self._pool = None
 
     async def _context_release(self):
@@ -736,26 +799,24 @@ class DBTransaction(DBClient):
 
         await self.rollback()
 
-    async def execute(self, query, *multiparams, **params):
+    async def _execute(self, query, *multiparams, **params):
 
         result = None
 
         if self._readonly:
             raise MySQLReadOnlyError()
 
-        async with self._lock:
+        try:
 
-            try:
+            conn = await self._get_conn()
 
-                conn = await self._get_conn()
+            result = await conn.execute(query, *multiparams, **params)
 
-                result = await conn.execute(query, *multiparams, **params)
+        except Exception as err:
 
-            except Exception as err:
+            await self._close_conn(True)
 
-                await self._close_conn(True)
-
-                raise err
+            raise err
 
         return result
 
