@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 
+__author__ = r'wsb310@gmail.com'
+
 import os
 import re
 import sys
@@ -43,6 +45,7 @@ from cachetools import cached, TTLCache
 from zipfile import ZipFile, ZIP_DEFLATED
 
 from stdnum import luhn
+from dateutil.parser import parse as date_parse
 
 from .error import BaseError
 
@@ -84,6 +87,8 @@ class Utils:
     getpid = staticmethod(os.getpid)
     getppid = staticmethod(os.getppid)
 
+    date_parse = staticmethod(date_parse)
+
     @classmethod
     def environment(cls):
 
@@ -123,6 +128,17 @@ class Utils:
             r'Expected str, bytes, or None; got %r' % type(val)
         )
 
+    @classmethod
+    def vague_string(cls, val, vague_str=r'*', vague_len=0.5, min_vague=1, max_vague=10):
+
+        val_len = len(val)
+        vague_len = max(min_vague, min(max_vague, cls.math.ceil(val_len * vague_len)))
+
+        r_show = (val_len - vague_len) // 2
+        l_show = max(1, val_len - r_show - vague_len) if val_len > min_vague else 0
+
+        return val[:l_show] + vague_str * vague_len + val[val_len - r_show:]
+
     @classmethod
     def json_encode(cls, val, **kwargs):
 
@@ -230,6 +246,43 @@ class Utils:
 
         return result
 
+    @classmethod
+    def split_float(cls, val, sep=r',', minsplit=0, maxsplit=-1):
+
+        result = [
+            float(item.strip()) for item in val.split(sep, maxsplit)
+            if cls.re_match(r'[\d\.]+', item.strip().lstrip(r'-'))
+        ]
+
+        fill = minsplit - len(result)
+
+        if fill > 0:
+            result.extend(0 for _ in range(fill))
+
+        return result
+
+    @classmethod
+    def join_float(cls, iterable, sep=r','):
+
+        result = []
+
+        for item in iterable:
+
+            cls = type(item)
+
+            if cls is float:
+
+                result.append(str(item))
+
+            elif cls is str:
+
+                item = item.strip()
+
+                if cls.re_match(r'[\d\.]+', item):
+                    result.append(item)
+
+        return sep.join(result)
+
     @classmethod
     def split_int(cls, val, sep=r',', minsplit=0, maxsplit=-1):
 
@@ -707,26 +760,26 @@ class Utils:
         return hashlib.sha512(val).hexdigest()
 
     @classmethod
-    def hmac_md5(cls, key, msg):
+    def hmac_md5(cls, key, msg, b64_std=False):
 
         key = cls.utf8(key)
         msg = cls.utf8(msg)
 
         result = hmac.new(key, msg, r'md5').digest()
 
-        result = base64.b64encode(result)
+        result = cls.b64_encode(result, b64_std)
 
         return cls.basestring(result)
 
     @classmethod
-    def hmac_sha1(cls, key, msg):
+    def hmac_sha1(cls, key, msg, b64_std=False):
 
         key = cls.utf8(key)
         msg = cls.utf8(msg)
 
         result = hmac.new(key, msg, r'sha1').digest()
 
-        result = base64.b64encode(result)
+        result = cls.b64_encode(result, b64_std)
 
         return cls.basestring(result)
 
