@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 
+__author__ = r'wsb310@gmail.com'
+
 
 # 基础异常
 class BaseError(Exception):
@@ -25,6 +27,11 @@ class MySQLReadOnlyError(BaseError):
     pass
 
 
+# 数据库客户端已销毁
+class MySQLClientDestroyed(BaseError):
+    pass
+
+
 # 常量设置异常
 class ConstError(BaseError):
     pass
