@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
 
+__author__ = r'wsb310@gmail.com'
+
+import os
 import queue
 import logging
 
@@ -108,9 +111,13 @@ class QueuedFileSink(FileSink):
                 pass
 
 
+DEFAULT_LOG_FILE_NAME = r'runtime_{time}.log'
+
+
 def init_logger(
         level, handler=None,
-        file_path=None, file_rotation=DEFAULT_LOG_FILE_ROTATOR, file_retention=0xff,
+        file_path=None, file_name=DEFAULT_LOG_FILE_NAME,
+        file_rotation=DEFAULT_LOG_FILE_ROTATOR, file_retention=0xff,
         debug=False
 ):
 
@@ -129,9 +136,12 @@ def init_logger(
             )
 
         if file_path:
+
+            _file_name, _file_ext_name = os.path.splitext(file_name)
+
             Utils.log.add(
                 QueuedFileSink(
-                    Utils.path.join(file_path, f'runtime_{{time}}_{Utils.getpid()}.log'),
+                    Utils.path.join(file_path, _file_name + '.pid-' + str(Utils.getpid()) + _file_ext_name),
                     rotation=file_rotation,
                     retention=file_retention
                 ),
